# Generated by Django 4.1.5 on 2023-01-20 22:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('scheduler', '0006_auto_20230118_1640'),
    ]

    operations = [
        migrations.AddField(
            model_name='cronjob',
            name='result_ttl',
            field=models.IntegerField(blank=True,
                                      help_text='The TTL value (in seconds) of the job result. -1: Result never expires, you should delete jobs manually. 0: Result gets deleted immediately. >0: Result expires after n seconds.',
                                      null=True, verbose_name='result ttl'),
        ),
        migrations.AlterField(
            model_name='cronjob',
            name='queue',
            field=models.CharField(help_text='Queue name', max_length=16, verbose_name='queue'),
        ),
        migrations.AlterField(
            model_name='repeatablejob',
            name='queue',
            field=models.CharField(help_text='Queue name', max_length=16, verbose_name='queue'),
        ),
        migrations.AlterField(
            model_name='scheduledjob',
            name='queue',
            field=models.CharField(help_text='Queue name', max_length=16, verbose_name='queue'),
        ),
    ]
