# Generated by Django 4.1.7 on 2023-02-19 22:12

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('scheduler', '0007_add_result_ttl'),
    ]

    operations = [
        migrations.RenameField(
            model_name='jobarg',
            old_name='str_val',
            new_name='val',
        ),
        migrations.RenameField(
            model_name='jobkwarg',
            old_name='str_val',
            new_name='val',
        ),
        migrations.RemoveField(
            model_name='jobarg',
            name='bool_val',
        ),
        migrations.RemoveField(
            model_name='jobarg',
            name='datetime_val',
        ),
        migrations.RemoveField(
            model_name='jobarg',
            name='int_val',
        ),
        migrations.RemoveField(
            model_name='jobkwarg',
            name='bool_val',
        ),
        migrations.RemoveField(
            model_name='jobkwarg',
            name='datetime_val',
        ),
        migrations.RemoveField(
            model_name='jobkwarg',
            name='int_val',
        ),
        migrations.AlterField(
            model_name='jobarg',
            name='arg_type',
            field=models.CharField(choices=[('str_val', 'string'), ('int_val', 'int'), ('bool_val', 'boolean'),
                                            ('datetime_val', 'Datetime'), ('callable', 'Callable')], default='str_val',
                                   max_length=12, verbose_name='Argument Type'),
        ),
        migrations.AlterField(
            model_name='jobkwarg',
            name='arg_type',
            field=models.CharField(choices=[('str_val', 'string'), ('int_val', 'int'), ('bool_val', 'boolean'),
                                            ('datetime_val', 'Datetime'), ('callable', 'Callable')], default='str_val',
                                   max_length=12, verbose_name='Argument Type'),
        ),
    ]
