# Generated by Django 4.2 on 2023-04-18 19:08

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('scheduler', '0011_worker_alter_queue_options_alter_cronjob_at_front_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cronjob',
            name='name',
            field=models.CharField(help_text='Name of the job.', max_length=128, unique=True, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='repeatablejob',
            name='name',
            field=models.CharField(help_text='Name of the job.', max_length=128, unique=True, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='scheduledjob',
            name='name',
            field=models.CharField(help_text='Name of the job.', max_length=128, unique=True, verbose_name='name'),
        ),
    ]
