# Generated by Django 5.1b1 on 2024-06-29 14:21

import scheduler.models.scheduled_task
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0017_remove_crontask_repeat_crontask_failed_runs_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='crontask',
            name='queue',
            field=models.CharField(choices=scheduler.models.scheduled_task.get_queue_choices, help_text='Queue name', max_length=255, verbose_name='queue'),
        ),
        migrations.AlterField(
            model_name='repeatabletask',
            name='queue',
            field=models.CharField(choices=scheduler.models.scheduled_task.get_queue_choices, help_text='Queue name', max_length=255, verbose_name='queue'),
        ),
        migrations.AlterField(
            model_name='scheduledtask',
            name='queue',
            field=models.CharField(choices=scheduler.models.scheduled_task.get_queue_choices, help_text='Queue name', max_length=255, verbose_name='queue'),
        ),
    ]
