# Generated by Django 5.0.1 on 2024-01-10 17:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0016_rename_jobarg_taskarg_rename_jobkwarg_taskkwarg_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='crontask',
            name='repeat',
        ),
        migrations.AddField(
            model_name='crontask',
            name='failed_runs',
            field=models.PositiveIntegerField(default=0, help_text='Number of times the task has failed', verbose_name='failed runs'),
        ),
        migrations.AddField(
            model_name='crontask',
            name='last_failed_run',
            field=models.DateTimeField(blank=True, help_text='Last time the task has failed', null=True, verbose_name='last failed run'),
        ),
        migrations.AddField(
            model_name='crontask',
            name='last_successful_run',
            field=models.DateTimeField(blank=True, help_text='Last time the task has succeeded', null=True, verbose_name='last successful run'),
        ),
        migrations.AddField(
            model_name='crontask',
            name='successful_runs',
            field=models.PositiveIntegerField(default=0, help_text='Number of times the task has succeeded', verbose_name='successful runs'),
        ),
        migrations.AddField(
            model_name='repeatabletask',
            name='failed_runs',
            field=models.PositiveIntegerField(default=0, help_text='Number of times the task has failed', verbose_name='failed runs'),
        ),
        migrations.AddField(
            model_name='repeatabletask',
            name='last_failed_run',
            field=models.DateTimeField(blank=True, help_text='Last time the task has failed', null=True, verbose_name='last failed run'),
        ),
        migrations.AddField(
            model_name='repeatabletask',
            name='last_successful_run',
            field=models.DateTimeField(blank=True, help_text='Last time the task has succeeded', null=True, verbose_name='last successful run'),
        ),
        migrations.AddField(
            model_name='repeatabletask',
            name='successful_runs',
            field=models.PositiveIntegerField(default=0, help_text='Number of times the task has succeeded', verbose_name='successful runs'),
        ),
    ]
