# Generated by Django 4.1.5 on 2023-01-18 16:40

from django.db import migrations


def forwards_func(apps, schema_editor):
    pass


def reverse_func(apps, schema_editor):
    # forwards_func() creates two Country instances,
    # so reverse_func() should delete them.
    cronjob_model = apps.get_model(app_label='scheduler', model_name='CronJob')
    db_alias = schema_editor.connection.alias
    cronjob_model.objects.using(db_alias).filter(name='Job scheduling jobs').delete()


class Migration(migrations.Migration):
    dependencies = [
        ('scheduler', '0005_alter_cronjob_at_front_alter_repeatablejob_at_front_and_more'),
    ]

    operations = [
        migrations.RunPython(forwards_func, reverse_func),
    ]
