# Generated by Django 4.2.5 on 2023-09-08 20:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("scheduler", "0013_alter_cronjob_queue_alter_repeatablejob_queue_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cronjob",
            name="created",
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name="cronjob",
            name="modified",
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name="repeatablejob",
            name="created",
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name="repeatablejob",
            name="modified",
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name="scheduledjob",
            name="created",
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name="scheduledjob",
            name="modified",
            field=models.DateTimeField(auto_now=True),
        ),
    ]
